/*************************************************************************
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005-2008 by Kohei Yoshida.
 *    1039 Kingsway Dr., Apex, NC 27502, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

#include "numeric/nlpbase.hxx"
#include "numeric/nlpmodel.hxx"
#include "numeric/funcobj.hxx"

using ::std::vector;

namespace scsolver { namespace numeric { namespace nlp {

struct BaseAlgorithmImpl
{
	Model* pModel;
    vector<double> Solution;
    bool Debug;

    BaseAlgorithmImpl() :
        pModel(NULL),
        Debug(false)
    {
    }
};

BaseAlgorithm::BaseAlgorithm() : m_pImpl( new BaseAlgorithmImpl )
{
}

BaseAlgorithm::~BaseAlgorithm()
{
}

Model* BaseAlgorithm::getModel() const
{
	return m_pImpl->pModel;
}

void BaseAlgorithm::setModel( Model* model )
{
	m_pImpl->pModel = model;
}

void BaseAlgorithm::getSolution(vector<double>& rVars) const
{
    getModel()->getFuncObject()->getVars(rVars);
}

void BaseAlgorithm::setDebug(bool debug)
{
    m_pImpl->Debug = debug;
}

bool BaseAlgorithm::isDebug() const
{
    return m_pImpl->Debug;
}

}}}

